#Скрипт для создания трех заданий планировщика для выполнения задач на восходе и закате
#http://www.outsidethebox.ms/20351

#Параметры запуска скрипта по умолчанию
##################################################################################
param(
  #Путь к папке со скриптами
  $path = (Get-Location).path,
  #ИД пользователя в формате "Имя_компьютера\Имя_пользователя"
  #Пользователь, запускающий этот скрипт:
  $user = [System.Security.Principal.WindowsIdentity]::GetCurrent().Name
)

#Общие переменные
##################################################################################
#ИД системы
$system = "NT AUTHORITY\SYSTEM"

#Задание 1 - "Sunrise-Sunset-Time"
#Выполнение скрипта sunrise_sunset_time.ps1 для получения времени восхода и заката, обновления заданий 2 и 3
##################################################################################

#Создание задания
##################################################################################
$taskname = "Sunrise-Sunset-Time" 
#Общие: выполнять с наивысшими правами от имени системы вне зависимости от входа
$principal = New-ScheduledTaskPrincipal -UserId $system -LogonType ServiceAccount
#Триггеры
$trigger = @(
    #Запускать ежедневно сразу после полуночи (не пробуждая ПК)
    $(New-ScheduledTaskTrigger -Daily -At 00:01),
	#Запускать при входе пользователя в систему
    $(New-ScheduledTaskTrigger -AtLogon)
)
#Параметры: запускать при работе от батареи; запускать немедленно если пропущено
$settings = New-ScheduledTaskSettingsSet -AllowStartIfOnBatteries -StartWhenAvailable
#Команда...
$execute = "powershell"
#... и ее параметры командной строки
$argument = "-ExecutionPolicy Bypass -WindowStyle Hidden -file $path\sunrise_sunset_time.ps1"
#Действие: "команда + параметры командной строки"
$action = New-ScheduledTaskAction -Execute $execute -Argument $argument
#Создать задание в планировщике
Register-ScheduledTask -TaskName $taskname -Action $action -Trigger $trigger -Settings $settings -Principal $principal

#Добавление триггера запуска по событию выхода из сна/гибернации
#У командлета New-ScheduledTaskTrigger нет параметра для событий ¯\_(ツ)_/¯
#По мотивам https://xplantefeve.io/posts/SchdTskOnEvent
##################################################################################
#Формирование триггера по событию в журнале
$class = cimclass MSFT_TaskEventTrigger root/Microsoft/Windows/TaskScheduler
$triggerevent = $class | New-CimInstance -ClientOnly
$triggerevent.Enabled = $true
$triggerevent.Subscription = "<QueryList><Query Id='0' Path='System'><Select Path='System'>*[System[Provider[@Name='Microsoft-Windows-Power-Troubleshooter'] and EventID=1]]</Select></Query></QueryList>"
#Изменение задания: получение текущих триггеров и добавление нового
Set-ScheduledTask -TaskName $taskname -Trigger $((Get-ScheduledTask -TaskName $taskname).triggers + $triggerevent)

#Задание 2 - "Sunrise"
#Выполнение скрипта sunrise.ps1 на восходе
##################################################################################
$taskname = "Sunrise" 
#Выполнять с наивысшими правами от имени текущего пользователя
$principal = New-ScheduledTaskPrincipal -UserId $user -RunLevel Highest
#Запускать однократно, дальше время будет обновлять задание 1
$trigger = New-ScheduledTaskTrigger -Once -At 00:01
#Значения $settings и $execute такие же, как в задании 1, поэтому здесь не дублируются
$argument = "-ExecutionPolicy Bypass -WindowStyle Hidden -file $path\sunrise.ps1"
$action = New-ScheduledTaskAction -Execute $execute -Argument $argument
Register-ScheduledTask -TaskName $taskname -Action $action -Trigger $trigger -Settings $settings -Principal $principal

#Задание 3 - "Sunset"
#Выполнение скрипта sunset.ps1 на закате
##################################################################################
$taskname = "Sunset" 
#Значения $trigger и $principal - в задании 2, а остальных переменных - в задании 1, поэтому здесь не дублируются
$argument = "-ExecutionPolicy Bypass -WindowStyle Hidden -file $path\sunset.ps1"
$action = New-ScheduledTaskAction -Execute $execute -Argument $argument
Register-ScheduledTask -TaskName $taskname -Action $action -Trigger $trigger -Settings $settings -Principal $principal

#Первичное выполнение задания "Sunrise-Sunset-Time"
##################################################################################
Start-ScheduledTask -TaskName "Sunrise-Sunset-Time"